#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: __init__.py
@time: 2019-02-08
"""

import click

from .settings import settings
from .train import TrainTable


@click.command()
@click.version_option()
@click.option("-f", "--from-station", prompt="请输入出发地", help="出发地")
@click.option("-t", "--to-station", prompt="请输入目的地", help="目的地")
@click.option("-d", "--date", prompt="请输入日期（YYYY-MM-DD）", help="日期")
@click.option("-s", "--seats", help="限制座位，如：一等座 二等座 无座")
@click.option("-n", "--trains-no", help="限制车次，如：G1 G2 G3")
@click.option("-z", "--zmode", default=False, is_flag=True, help="高级模式，查询中间站点")
@click.option("-zz", "--zzmode", default=False, is_flag=True, help="终极模式，查询所有中间站点")
@click.option("-r", "--remaining", default=False, is_flag=True, help="只看有票")
@click.option("-v", "--verbose", default=False, is_flag=True, help="调试模式")
@click.option("--gcd", default=False, is_flag=True, help="只看高铁动车城际")
@click.option("--ktz", default=False, is_flag=True, help="只看普快特快直达等")
@click.option("--proxies-file", help="代理列表文件")
@click.option("--stations-file", help="站点信息文件")
@click.option("--cdn-file", help="CDN文件")
def main(
    from_station,
    to_station,
    date,
    seats,
    trains_no,
    zmode,
    zzmode,
    remaining,
    verbose,
    gcd,
    ktz,
    proxies_file,
    stations_file,
    cdn_file,
):
    """
    12306查票助手 https://github.com/0xHJK/x12306

    Example: python3 x12306.py -f 上海 -t 北京 -d "2024-05-01"
    """
    settings.update(
        fs=from_station,
        ts=to_station,
        date=date,
        seats=seats,
        trains_no=trains_no,
        zmode=zmode,
        zzmode=zzmode,
        remaining=remaining,
        verbose=verbose,
        gcd=gcd,
        ktz=ktz,
        proxies_file=proxies_file,
        stations_file=stations_file,
        cdn_file=cdn_file,
    )

    print("\n-----------------------")
    print(settings)
    print("查询中...请稍等... :)\n")
    tt = TrainTable()
    tt.update()
    tt.echo()



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: train.py
@time: 2019-02-08
"""
"""
    搜索结果中的班次信息
"""

import time
import re
import requests
import prettytable as pt

from .settings import settings
from .utils import colorize


class Train:
    """
    搜索结果中的班次信息，包括车次号、余票、时间等信息
    """

    def __init__(self):
        self.full_no = ""  # 编号全称
        self.no = ""  # 编号简称
        self._fs_code = ""
        self._ts_code = ""
        self._fs = ""
        self._ts = ""
        self.start_time = ""
        self.end_time = ""
        self.duration = ""
        self.remaining = []

    def __str__(self):
        colored_remaining = [
            colorize(s, "green") if s != "无" and s != "--" else s
            for s in self.remaining
        ]
        remaining_seats = "/".join(colored_remaining)
        if re.match("[GCD]", self.no):
            colored_no = colorize(self.no, self.no[0].lower())
        else:
            colored_no = colorize(self.no, "o")
        s = "{:^5} | {:^5} | {:\u3000<4} | {:\u3000<4} | {:^5} | {} | {:^5}".format(
            colored_no,
            self.start_time,
            self._fs,
            self._ts,
            self.end_time,
            remaining_seats,
            self.duration,
        )
        return s

    def __eq__(self, other):
        """车次、出发站和目的地相同判断为相等"""
        return (
            self.full_no == other.full_no
            and self._fs == other._fs
            and self._ts == other._ts
        )

    def __lt__(self, other):
        """根据车次号排序"""
        return self.start_time < other.start_time

    def __gt__(self, other):
        """根据车次号排序"""
        return self.start_time > other.start_time

    @property
    def fs_code(self):
        return self._fs_code

    @fs_code.setter
    def fs_code(self, fs_code):
        self._fs_code = fs_code
        self._fs = settings.reverse_stations_dict.get(fs_code, "")

    @property
    def ts_code(self):
        return self._ts_code

    @ts_code.setter
    def ts_code(self, ts_code):
        self._ts_code = ts_code
        self._ts = settings.reverse_stations_dict.get(ts_code, "")

    @property
    def has_remaining(self) -> bool:
        for r in self.remaining:
            if r != "无" and r != "--":
                return True
        return False

    @property
    def row(self) -> list:
        """关键信息列表"""
        colored_remaining = [
            colorize(s, "green") if s != "无" and s != "--" else s
            for s in self.remaining
        ]
        remaining_seats = "/".join(colored_remaining)
        if re.match("[GCD]", self.no):
            colored_no = colorize(self.no, self.no[0].lower())
        else:
            colored_no = colorize(self.no, "o")
        return [
            colored_no,
            self.start_time,
            self._fs,
            self._ts,
            self.end_time,
            remaining_seats,
            self.duration,
        ]


class TrainTable:
    """
    搜索结果列表
    """

    def __init__(self):
        self.trains_list = []
        self._session = requests.Session()
        self._session.headers.update(settings.headers)

    @property
    def session(self):
        # TODO: 设置代理、随机headers等
        return self._session

    def echo(self):
        """
        对外调用的方法，用来打印查询结果
        """
        tb = pt.PrettyTable()
        tb.field_names = ["车次", "发车", "出发站", "到达站", "到达", "余票", "历时"]
        self.cleanup()
        for train in self.trains_list:
            tb.add_row(train.row)
        print(tb)

    def cleanup(self):
        """处理trains_list，排序和删除无效数据"""
        t_list = []
        if settings.remaining:
            for train in self.trains_list:
                # print(train.no, train.remaining, train.has_remaining)
                if train.has_remaining:
                    t_list.append(train)
        else:
            t_list = self.trains_list
        self.trains_list = sorted(t_list)

    def update(self):
        """
        对外调用的方法，查询12306列车信息，把结果更新到trains_list中
        """
        if settings.zmode:
            self.trains_list = self._query_trains_zmode(
                settings.fs_code,
                settings.ts_code,
                settings.date,
                settings.trains_no_list,
            )
        else:
            self.trains_list = self._query_trains(
                settings.fs_code,
                settings.ts_code,
                settings.date,
                settings.trains_no_list,
            )

    def _query(self, url, params, retries=1) -> dict:
        """
            查询方法，根据url和参数，返回json对象
        :param url: 查询url
        :param params: 查询参数
        :param retries: 重试次数
        :return: 返回json对象
        """
        s = self.session
        # print("查询中...", " ".join([v for v in params.values()]))
        if settings.verbose:
            print(url, params)
        try:
            r = s.get(url, params=params, timeout=settings.timeout)
            j = r.json()
        except Exception as e:
            print(colorize("第 %i 查询失败" % retries, "red"), e)
            print(url, params)
            print(s.headers)
            if retries < settings.max_retries:
                time.sleep(retries * settings.timeout)
                return self._query(url, params, retries + 1)
            else:
                j = {}
        return j

    def _query_stations(self, train) -> list:
        """
            查询沿途车站信息
        :param train: Train对象
        """
        stations_list = []
        # 准备请求参数
        params = {
            "train_no": train.full_no,
            "from_station_telecode": train.fs_code,
            "to_station_telecode": train.ts_code,
            "depart_date": settings.date,
        }
        j = self._query(settings.trainno_url, params)
        if j and j.get("data") and j["data"].get("data"):
            for item in j["data"]["data"]:
                if item["isEnabled"]:
                    stations_list.append(item["station_name"])
            stations_list = stations_list[1:-1]  # 除了出发站和目标站本身
        return stations_list

    def _query_trains(self, fs_code, ts_code, date, trains_no_list) -> list:
        """
            普通查询方法，根据出发地和目的地编码、日期和限制车次，返回trains列表
            仅被内部调用，调用前处理好参数
        :param fs_code: 出发地编码
        :param ts_code: 目的地编码
        :param date: 日期
        :param trains_no_list: 选择车次列表
        :return: 返回搜索结果列表，每一项是一个Train对象
        """
        # 准备请求参数
        params = {
            "leftTicketDTO.train_date": date,
            "leftTicketDTO.from_station": fs_code,
            "leftTicketDTO.to_station": ts_code,
            "purpose_codes": "ADULT",
        }
        trains_list = []
        j = self._query(settings.query_url, params)
        if j and j.get("data") and j["data"].get("result"):
            raws = j["data"]["result"]
            # 处理返回结果
            for raw in raws:
                fields = raw.split("|")
                # 如果限制了车次，且搜索车次不在目标车次中则丢弃
                if trains_no_list and fields[3] not in trains_no_list:
                    continue
                if settings.gcd and fields[3][0] not in "GCD":
                    # 只看高铁动车城际
                    continue
                if settings.ktz and fields[3][0] in "GCD":
                    # 只看普通列车
                    continue

                train = Train()
                train.full_no = fields[2]  # 编号全称
                train.no = fields[3]  # 编号简称
                train.fs_code = fields[6]
                train.ts_code = fields[7]
                train.start_time = fields[8]
                train.end_time = fields[9]
                train.duration = fields[10]
                for i in settings.seats_code_list:
                    train.remaining.append(fields[i] or "--")
                trains_list.append(train)
        return trains_list

    def _query_trains_zmode(self, fs_code, ts_code, date, trains_no_list) -> list:
        """
            高级查询模式，会查询从出发站到沿途所有站的车次情况
            仅被内部调用，调用前处理好参数
        :param fs_code: 出发地编码
        :param ts_code: 目的地编码
        :param date: 日期
        :param trains_no_list: 限制车次
        :return: Train对象列表
        """
        trains_list = self._query_trains(fs_code, ts_code, date, trains_no_list)
        trains_no_list = [train.no for train in trains_list]
        stations_list = []

        for train in trains_list:
            stations_list += self._query_stations(train)
        stations_list = list(set(stations_list))

        for station in stations_list:
            ts_code = settings.stations_dict.get(station, "")
            if ts_code:
                trains_list += self._query_trains(
                    fs_code, ts_code, date, trains_no_list
                )

        return trains_list



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
@author: HJK
@file: settings.py
@time: 2024-04-24
"""

"""
全局配置和共享变量，统一config和glovar
"""

import re
import sys
import platform
import hashlib
import secrets
import requests
from os import path

# 默认配置
SEAT_TYPES = {
    "特等座": 25,
    "商务座": 32,
    "一等座": 31,
    "二等座": 30,
    "高级软卧": 21,
    "软卧": 23,
    "动卧": 33,
    "硬卧": 28,
    "软座": 24,
    "硬座": 29,
    "无座": 26,
    "其他": 22,
}
DEFAULT_HEADERS = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate, br",
    "Accept-Language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7,ja;q=0.6",
    "Cache-Control": "no-cache",
    "Connection": "keep-alive",
    "Host": "kyfw.12306.cn",
    "Referer": "https://kyfw.12306.cn/otn/leftTicket/init",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_2) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/72.0.3626.96 Safari/537.36",
    "X-Requested-With": "XMLHttpRequest",
    "Cookie": "JSESSIONID=" + hashlib.md5(secrets.token_bytes(16)).hexdigest().upper(),
}

DEFAULT_SEATS = "一等座 二等座 无座"
DEFAULT_PROXIES_FILE = path.join(path.dirname(__file__), "data", "proxies.txt")
DEFAULT_STATIONS_FILE = path.join(path.dirname(__file__), "data", "stations.txt")
DEFAULT_CDN_FILE = path.join(path.dirname(__file__), "data", "cdn.txt")

DEFAULT_MAX_RETRIES = 3
DEFAULT_TIMEOUT = 5
DEFAULT_BASE_URL = "https://kyfw.12306.cn/otn/leftTicket"
DEFAULT_TRAINNO_URL = "https://kyfw.12306.cn/otn/czxx/queryByTrainNo"


class Singleton(type):
    _instances = {}

    def __call__(cls, *args, **kwargs):
        if cls not in cls._instances:
            cls._instances[cls] = super(Singleton, cls).__call__(*args, **kwargs)
        return cls._instances[cls]


class Settings(metaclass=Singleton):
    """
    全局配置和共享变量
    """

    def __init__(self):
        # 从哪里出发 -f --from-station
        self.fs = ""
        # 到哪里去 -t --to-station
        self.ts = ""
        # 日期 -d --date （格式：2019-01-01）
        self.date = ""
        # 车次代码 -n --trains-no （格式：G1 G2 G3）
        self.trains_no = ""
        # 座位 -s --seats （格式：一等座 二等座 无座）
        self.seats = DEFAULT_SEATS
        # 高级模式 -z --zmode （查询始发站往后到沿途的站点是否有票）
        self.zmode = False
        # 终极模式 -zz --zzmode （查询沿途所有站点之间是否有票）
        self.zzmode = False
        # 只看高铁动车城际 -g --gcd
        self.gcd = False
        # 只看普通列车（非gcd）-k --ktz
        self.ktz = False
        # 只看有余票 -r --remaining
        self.remaining = False
        # debug模式 -v --verbose
        self.verbose = False

        # 代理文件 --proxies-file （从文件中随机读取代理）
        self.proxies_file = DEFAULT_PROXIES_FILE
        # 站点信息文件 --stations-file （从12306下载）
        self.stations_file = DEFAULT_STATIONS_FILE
        # CDN文件 --cdn-file
        self.cdn_file = DEFAULT_CDN_FILE

        self.headers = DEFAULT_HEADERS
        self.init_url = DEFAULT_BASE_URL + "/init"
        self.query_url = DEFAULT_BASE_URL + "/query"
        self.trainno_url = DEFAULT_TRAINNO_URL
        self.timeout = DEFAULT_TIMEOUT
        self.max_retries = DEFAULT_MAX_RETRIES

        self._stations_dict = {}

    @property
    def stations_dict(self):
        # {站名：编码} 如 {北京：VAP}
        if self._stations_dict:
            return self._stations_dict
        else:
            self.update_stations()
            return self._stations_dict

    @property
    def reverse_stations_dict(self):
        # {编码：站名} 如 {VAP：北京}
        return {v: k for k, v in self.stations_dict.items()}

    @property
    def proxies_list(self):
        if self._proxies_list:
            return self._proxies_list
        else:
            self.update_proxies()
            return self._proxies_list

    @property
    def fs_code(self):
        return self.stations_dict.get(self.fs, "")

    @property
    def ts_code(self):
        return self.stations_dict.get(self.ts, "")

    @property
    def trains_no_list(self):
        return self.trains_no.split()

    @property
    def seats_list(self):
        # 获得座位列表
        return [s for s in self.seats.split() if s in SEAT_TYPES]

    @property
    def seats_code_list(self):
        # 获得座位编号列表
        return [SEAT_TYPES[s] for s in self.seats_list]

    def update(self, **kwargs):
        for k, v in kwargs.items():
            if v is not None and hasattr(self, k):
                setattr(self, k, v)

        self.update_stations()
        self.update_proxies()
        self.update_query_url()

    def update_stations(self):
        # 如果stations文件不存在则终止程序
        if not path.isfile(self.stations_file):
            print(self.stations_file)
            print("Stations file is not exsits.")
            sys.exit(-1)

        # 更新站点信息
        stations = open(self.stations_file, "r", encoding="utf-8").read()
        for station in stations.split("@"):
            if not station:
                continue
            t = station.split("|")
            # {站名：编码} 如 {北京：VAP}
            self._stations_dict[t[1]] = t[2]

    def update_proxies(self):
        # 更新代理信息
        if path.isfile(self.proxies_file):
            # 如果代理列表文件存在
            self._proxies_list = open(
                self.proxies_file, "r", encoding="utf-8"
            ).readlines()

    def update_query_url(self):
        try:
            r = requests.get(self.init_url, headers=self.headers, timeout=self.timeout)
            if r.status_code == 200:
                m = re.search(r"var CLeftTicketUrl = \'(.*?)\'", r.text)
                self.query_url = DEFAULT_BASE_URL + "/" + m.group(1).split("/")[1]
            else:
                print("Failed to update query url.")
                sys.exit(-1)
        except Exception as e:
            print(e)
            print("Failed to update query url.")
            sys.exit(-1)

    def __str__(self):
        """返回配置信息"""
        s1 = "出发地：%s | 目的地：%s | 日期：%s\n" % (
            self.highlight(self.fs),
            self.highlight(self.ts),
            self.highlight(self.date),
        )
        s2 = "限制车次：%s | 限制座位：%s\n" % (
            self.highlight(self.trains_no or "无"),
            self.highlight(self.seats),
        )
        s3 = "只看高铁动车：%s | 只看普通列车：%s\n" % (
            self.highlight(self.gcd),
            self.highlight(self.ktz),
        )
        s4 = "高级模式：%s | 终极模式：%s | 只看有票：%s\n" % (
            self.highlight(self.zmode),
            self.highlight(self.zzmode),
            self.highlight(self.remaining),
        )
        return s1 + s2 + s3 + s4

    def highlight(self, s):
        if platform.system() == "Windows":
            return str(s)
        return "\033[33m" + str(s) + "\033[0m"


settings = Settings()
